#include "Galaxy.h"
#include "ImageConfig.h" // IMG_ID
#include "RockFactory.h"
#include "GraphicsSystem.h"
#include "Entity.h"
#include "ImageSprite.h"
#include "Rock.h"
#include "BonusLife.h"
#include "ICollidableEntity.h"
#include "StarField.h"
#include "SoundConfig.h" // SFX_ID
#include "SoundSystem.h"
#include "AssertButOnlyInDebug.h"

enum { 
		NUMBER_OF_ROCKS = 24 
};

Galaxy::Galaxy():
m_imgSpriteRef(GraphicsSystem::GetSingleton()->GetImage(IMG_ID_GALAXY_BG1))
{	
	m_rocks = RockFactory::CreateNewMixedRocks(NUMBER_OF_ROCKS, *this); // creates them and links them to this galaxy

	m_bonusLife = new BonusLife(*this); 
	m_starField = new StarField();
}

Galaxy::~Galaxy()
{
	delete m_starField;
	m_starField = NULL;

	delete m_bonusLife;
	m_bonusLife = NULL;

	while (!m_rocks.empty())
	{
		delete m_rocks.back();
		m_rocks.pop_back();
	}

}

void Galaxy::ResetBeforeGame()
{
	Rock::ResetRockSpeed(); // back to start speed
}

void Galaxy::ResetBeforeLevel(int level) 
{	
	Rock::AllowSpawn();

	for (int i = 0; i < NUMBER_OF_ROCKS; i++)
		m_rocks[i]->ResetBeforeLevel(); 

	m_bonusLife->ResetBeforeLevel();
	ChangeBackground(level);	

}


void Galaxy::ChangeBackground(int level)
{
	assert(level > 0 && "Cannot have zero or negative index on background");

	IMG_ID bgIds[] = { IMG_ID_GALAXY_BG1, IMG_ID_GALAXY_BG2, IMG_ID_GALAXY_BG3, IMG_ID_GALAXY_BG4, IMG_ID_GALAXY_BG5, IMG_ID_GALAXY_BG6 };
	int numBGIds = sizeof (bgIds) / sizeof (bgIds[0]);

	assert(level <= numBGIds && "Too high index of background");	

	m_imgSpriteRef = GraphicsSystem::GetSingleton()->GetImage(bgIds[level-1]); // level starts with index 1, backgrounds with index 0

}


void Galaxy::Update(float elapsedTime, bool bIsStartOfLevel)
{
	Rock::ResetNrOfTrySpawnAttempts(bIsStartOfLevel); // don't spawn ALL rocks in the beginning, it's too hard too soon
		
	for (unsigned int i = 0; i < m_rocks.size(); i++)
		m_rocks[i]->Update(elapsedTime);

	m_bonusLife->Update(elapsedTime);
	m_starField->Update(elapsedTime);

}

void Galaxy::Draw()
{
	m_imgSpriteRef.Draw();
	
	m_starField->Draw();

	m_bonusLife->Draw();

	for (int i = 0; i < NUMBER_OF_ROCKS; i++)
		m_rocks[i]->Draw();

}



void Galaxy::StopSpawnRocks() 
{	
	Rock::StopSpawn();
}
void Galaxy::IncreaseRockSpeed()
{
	Rock::IncreaseRockSpeed();
}
bool Galaxy::IsAllowedToSpawnRocks() const
{ 
	return Rock::IsAllowedToSpawn();
}

void Galaxy::HandleCollisions(ICollidableEntity& playerRef)
{
	HandlePlayerVsBonusLifeCollisions(playerRef);
	HandlePlayerVsRockCollisions(playerRef);
}

void Galaxy::HandlePlayerVsBonusLifeCollisions(ICollidableEntity& playerRef)
{
	if (!m_bonusLife->IsAlive()) 
		return;

	if (playerRef.IsIntersecting( *m_bonusLife ))
	{
		SoundSystem::GetSingleton()->PlaySFX(SFX_ID_BONUS_LIFE);
		m_bonusLife->Unspawn();
		playerRef.CollidedWithBonusLife();
	}
}

void Galaxy::HandlePlayerVsRockCollisions(ICollidableEntity& playerRef)
{
	if( HasCollidedWithRock(playerRef) ) // hit a rock
	{	
		if(playerRef.CanCollideWithRocks())
		{
			SoundSystem::GetSingleton()->PlaySFX(SFX_ID_HIT);

			// time to die
			playerRef.CollidedWithRock();
		}
	}
}


bool Galaxy::HasCollidedWithRock(const IIntersectableEntity& entityRef) const
{
	for (int i = 0; i < NUMBER_OF_ROCKS; i++)
	{
		if ( m_rocks[i]->IsAlive() && &entityRef != m_rocks[i] ) // checking so entity is not comparing towards itself
		{
			if(entityRef.IsIntersecting( *m_rocks[i] ))
				return true;
		}
	}
	return false;
}