#pragma once //microsoft-specific

// disable warning C4290: "C++ exception specification ignored except to indicate a function is not __declspec(nothrow)"
// I usually don't turn off warnings. But this warning comes from Visual studio because the compiler "accepts but does not implement" specific throws.
#pragma warning( disable : 4290 ) // microsoft-specific

#include "SDL_opengl.h" // GLfloat
#include "Globals.h" // XAlignment, FontSize
#include "ImageConfig.h" // IMG_ID
#include "SDL.h" // Uint8
#include <vector>
#include "TextConfig.h" // TEXT_ID

// forward declaration to speed up compilation
class ImageSprite;
class ImageSubSystem;
class TextSubSystem;

/**
* System for handling graphics.
* Implemented as singleton to ensure only one instance exists which is globally available
* Facade towards subsystems for texts and images.
* Always call Init() first to initialize and load window, SDL, fonts and sprites.
* Has functions for drawing to screen buffer and flushing out buffer to screen. 
* Gives references to sprites by their enum id, and text arrays from filenames.
* 
* Clean up by calling Destroy() (preferrably from same class which called Init())
*/
class GraphicsSystem
{
public:

	// Returns pointer to instance of this class. Initializes instance on first call
	static GraphicsSystem*		GetSingleton();

	// Initializes window, graphics, fonts, sprites through subsystems
	// Return value of this function depends on if all font and image files were found.
	// Returns false if anything failed. Error handling should be done by caller.
	bool 						Init();

	// Gives out a reference to the sprite with given id.
	// Throws an error message if the sprite could not be retrieved
	ImageSprite& 				GetImage(IMG_ID id) throw (const char*);

	// Fills screen with black
	void 						ClearScreenBufferWithBlack();

	// Uses internal subsystem TextSubSystem to print given text to screen buffer with specified settings
	void 						DrawTextOntoScreenBuffer(const char *textToDisplay, GLfloat posY, XAlignment alignX, Uint8 textRed,  Uint8 textGreen,  Uint8 textBlue,  FontSize fontSize, float opacity);

	// Empties the buffer onto screen and shows new graphical content
	void 						ShowBufferedScreenOnDisplay();

	// Uses internal subsystem TextSubSystem:
	// Returns a vector with text lines from specified file. Will return an "missing text" placeholder in case of fail.
	// Memory is deallocated automatically when game exits
	std::vector<const char*>	ReadTextFromFile(TEXT_ID id);

	// Cleans up subsystems and allocated memory
	// Most operating systems delete static instances at program termination, but it's done here manually as good practice
	static void					Destroy();

private:
	// Avoid multiple instantiation.
								GraphicsSystem();

	// Encapsulated to only be used by Destroy() in order to delete in a consistent way.
								~GraphicsSystem(); 

	// Avoid copying of this class
								GraphicsSystem(const GraphicsSystem &); 

	// Avoid assign of this class
	GraphicsSystem				&operator=(const GraphicsSystem &); 

	// Initiates SDL, window, openGL and sets color mask for textures
	bool						InitGameWindow();
	
	// Static internal instance of this class
	static GraphicsSystem*		ms_graphics;

	// Subsystems
	ImageSubSystem*				m_imageSubSystem;
	TextSubSystem*				m_textSubSystem;	
};
