#include "ImageSprite.h"
#include "SDL_image.h"
#include "MathUtil.h"
#include "AssertButOnlyInDebug.h"

ImageSprite::ImageSprite(const char* filePath)
{
	assert(filePath && "ImageSprite::ImageSprite(const char* filePath): Filename entered for imagesprite is null");
	assert(strlen(filePath) > 0 && "ImageSprite::ImageSprite(const char* filePath): Filename for imagesprite is empty");
	strcpy_s(m_fileName, filePath);
}

ImageSprite::~ImageSprite()
{
	if (m_OGL_Texture != NULL)  
		glDeleteTextures(1, &m_OGL_Texture);
}

bool ImageSprite::Load()
{
	// Temp variables
	SDL_Surface *tempSurface;

	if( (tempSurface = IMG_Load(m_fileName)) == NULL ) 
	{
		printf("ERROR: %s: Failed to load image: IMG_Load: %s\n", __FUNCTION__, SDL_GetError()); // microsoft-specific
		assert(false && "ImageSprite::Load(): Failed to load an image. Check log file for details.");
		return false;
	}

	// Went fine, continuing...
	GLenum textureFormat = 0;
	GLint nrOfColors = tempSurface->format->BytesPerPixel;

	if (nrOfColors == 4) // alpha
	{
		textureFormat = GL_RGBA;
	}
	else if (nrOfColors == 3)
	{
		textureFormat = GL_RGB;
	}
	else
	{
		printf("ERROR: %s: Failed to format texture: image is not in TrueColor\n", __FUNCTION__); // microsoft-specific
		SDL_FreeSurface(tempSurface);
		return false;	
	}

	// standard settings
	m_screenX = 0.0f; 
	m_screenY = 0.0f; 
	m_rotationDegree = 0.0f;
	m_opacity = 1.0f;
	m_redHue = 1.0f;
	m_greenHue = 1.0f;
	m_blueHue = 1.0f;
	m_bSmooth = true;
	m_width = tempSurface->w;
	m_height = tempSurface->h;
	m_OGL_Texture = (GLuint)NULL;
	m_origoX = 0.0f;
	m_origoY = 0.0f;

	// setting dimensions to be current or next power of two. some gfx drives can only handle that
	m_width = MathUtil::NextPowerOfTwo(tempSurface->w);
	m_height = MathUtil::NextPowerOfTwo(tempSurface->h); 

	// Went fine, continuing... 
	glGenTextures( 1, &m_OGL_Texture );

	glBindTexture( GL_TEXTURE_2D, m_OGL_Texture );

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

	// creating texture and copying pixels into it (need power of two for some gfx cards)
	glTexImage2D(GL_TEXTURE_2D, 0, nrOfColors, (GLsizei) m_width, (GLsizei) m_height, 0, 
					textureFormat, GL_UNSIGNED_BYTE, tempSurface->pixels); 

	SDL_FreeSurface(tempSurface);

	return true;
}

void ImageSprite::Draw() 
{
    glPushMatrix();  // pushing current matrix onto the stack
    glBindTexture(GL_TEXTURE_2D, m_OGL_Texture);
    glEnable(GL_TEXTURE_2D);

	if (m_rotationDegree > 0.0f) // translating coordinates to the origo in order to rotate around that point
	{
		glTranslatef(m_screenX + m_origoX, m_screenY + m_origoY, 0.0f);
	    glRotatef(m_rotationDegree, 0.0f, 0.0f, 1.0f);
		glTranslatef(- (m_screenX + m_origoX) , - (m_screenY + m_origoY), 0.0f); // resetting, putting inverted values as delta
	}
	
	glTranslatef(m_screenX, m_screenY, 0.0f);

    glColor4f(m_redHue, m_greenHue, m_blueHue, m_opacity);

    if (m_bSmooth)
    {
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    }

    glBegin(GL_QUADS);  // start push quads through the pipeline
        glTexCoord2i(0, 0); // upper left corner point
        glVertex3f(0.0f, 0.0f, 0.0f);  // is here in texture

        glTexCoord2i(1, 0);
        glVertex3f((GLfloat) m_width, 0.0f, 0.0f);

        glTexCoord2i(1, 1);
        glVertex3f((GLfloat) m_width, (GLfloat) m_height, 0.0f);

        glTexCoord2i(0, 1);
        glVertex3f( 0.0f, (GLfloat) m_height, 0.0f);

    glEnd();

    glPopMatrix(); // restore the matrix from the stack
}
int ImageSprite::GetHeight() const
{ 
	return m_height; 
}
int ImageSprite::GetWidth() const
{ 
	return m_width; 
}
float ImageSprite::GetOpacity() const
{ 
	return m_opacity; 
}
float ImageSprite::GetScreenX() const
{
	return m_screenX;
}
float ImageSprite::GetScreenY() const
{
	return m_screenY;
}
void ImageSprite::SetOpacity(float opacity) 
{ 
	if(opacity < 0.0f) // cannot be negative
		opacity = 0.0f;
	else if (opacity > 1.0f) // max 1
		opacity = 1.0f;

	m_opacity = opacity; 
}
void ImageSprite::SetWidth(int width) 
{ 
	assert(width > 0 && "ImageSprite::SetWidth(int width): width must be a positive value above zero!");
	m_width = width; 
}
void ImageSprite::SetHeight(int height) 
{ 
	assert(height > 0 && "ImageSprite::SetHeight(int height): height must be a positive value above zero!");
	m_height = height; 
}
void ImageSprite::SetRotationDegree(float rotationDegree)
{
	m_rotationDegree = rotationDegree;
}
void ImageSprite::SetOrigo(float origoX, float origoY)
{
	m_origoX = origoX;
	m_origoY = origoY;
}
void ImageSprite::SetScreenX(float screenX)
{
	m_screenX = screenX;
}
void ImageSprite::SetScreenY(float screenY)
{
	m_screenY = screenY;
}
