#include "ImageSubSystem.h"
#include "ImageSprite.h"
#include "AssertButOnlyInDebug.h"

// X-macro for mapping image resources (enum) together with its path string. 
// Defining the paths below. The rest of the Macro is in ImageConfig.h
// This part of the code is placed here to be defined and compiled only once
#define X(a, b) b,
	const char *path_for_id[] = {
	  IMG_ID_AND_PATH_TABLE
	};
#undef X


ImageSubSystem::ImageSubSystem()
{
}

ImageSubSystem::~ImageSubSystem()
{
	printf("INFO: Destroying ImageSubSystem\n");
	UnloadImages();
}

bool ImageSubSystem::Init()
{	
	if (!AddImages() || !LoadImages()) 
		return false;
	return true;
}

bool ImageSubSystem::AddImages()
{
	// number of textures = length of array
	int max = sizeof(path_for_id) / sizeof (path_for_id[0]);

	for(int i = 0; i < max; i++) 
	{
		if(!AddImage(IMG_ID(i)))
		{
			printf("ERROR: %s: Failed to add all imagesprites\n", __FUNCTION__); // microsoft-specific
			return false; // don't play the game without all the images
		}
	}
	return true;
}

bool ImageSubSystem::AddImage(IMG_ID mapKeyId)
{
	ImageSprite* imgSprite = new ImageSprite(path_for_id[mapKeyId]); 

	std::pair <std::map <IMG_ID, ImageSprite*>::iterator, bool > res;
	res = m_id_to_sprites.insert( std::pair<IMG_ID, ImageSprite*>( mapKeyId, imgSprite) ); // trying to put sprite in map

	if (!res.second) { 	// the sprite was not added since the key id already existed in the map
		// programmer error, not runtime error, so handled by assert
		printf("ERROR: %s: Failed to add image: the id with filepath %s has already been added\n", __FUNCTION__, path_for_id[mapKeyId]); // microsoft-specific
		assert(false && " ImageSubSystem::AddImage(ID mapKeyId): Failed to add image since its id has already been added. See logfile for image path details."); 
		return false; //graceful shutdown
	}	
	// else: new sprite did not exist in map and was added to it

	return true;
}



bool ImageSubSystem::LoadImages()
{
	std::pair <std::map <IMG_ID, ImageSprite*>::iterator, bool > res;

	for(std::map<IMG_ID, ImageSprite*>::iterator map_iter = m_id_to_sprites.begin(); map_iter != m_id_to_sprites.end(); map_iter++)
	{
		ImageSprite* imgSprite = map_iter->second;
		if (!imgSprite->Load())  // image sprites are loaded in from within themselves since they own their gfx variables such as witdh			
			return false; 
	}

	printf("INFO: Successfully loaded all images\n");
	return true;
}


void ImageSubSystem::UnloadImages()
{
	for(std::map<IMG_ID, ImageSprite*>::iterator map_iter = m_id_to_sprites.begin(); map_iter != m_id_to_sprites.end(); map_iter++)
		delete map_iter->second;
	
	if (!m_id_to_sprites.empty()) 
	{
		m_id_to_sprites.clear();
		printf("INFO: Successfully unloaded all imagesprites\n");
	}
}

ImageSprite& ImageSubSystem::GetImage(IMG_ID id) // function will be called runtime 
{
	std::map<IMG_ID, ImageSprite*>::iterator res;
	res = m_id_to_sprites.find(id);
	if (res == m_id_to_sprites.end()) // if reached beyond end of map
	{
		// This should be a programmer error, so handled by assert. 
		// In case it remains in Release mode, assert will be ignored and a recover mechanism will trigger helow. 
		// If it fails to recover, it will throw an exception and program should exit. 
		assert(false && "ImageSubSystem::GetImage(ID id): Could not find sprite with given ID"); // will only be shown in debug
		printf("ERROR: %s: Failed to retrieve image: the id with filepath %s can not be found. Trying to recover by adding and loading image\n", __FUNCTION__, path_for_id[id]); // to log. microsoft-specific

		// Try to recover by adding missing image
		if(!AddImage(id) || !m_id_to_sprites[id]->Load())
		{
			printf("ERROR: %s: Failed to add and load imagesprite with filepath %s\n", __FUNCTION__, path_for_id[id]); // microsoft-specific
			throw("ImageSubSystem::GetImage(ID id): could not retrieve nor recover and add & load an image");	// only throw error if something is runtime unrecoverable. Program will exit. User should get some form of feedback		
		} 
		else //the id should be added and returned below
			printf("INFO: Image added and loaded successfully\n");
	}
	return *m_id_to_sprites[id]; 
}
