#pragma once //microsoft-specific

// disable warning C4290: "C++ exception specification ignored except to indicate a function is not __declspec(nothrow)"
// I usually don't turn off warnings. But this warning comes from Visual studio because the compiler "accepts but does not implement" specific throws.
#pragma warning( disable : 4290 ) // microsoft-specific

#include <map>
#include "ImageConfig.h" // IMG_ID

class ImageSprite; // forward declaration to speed up compilation

/**
* Resource management system for handling ImageSprites. All ImageSprites are loaded and unloaded from here.
* Always call Init() first to initialize and load textures into the internal map of ImageSprites.
* This should be done at start of program.
*
* This class has functions for retriving an imagesprite with given id. 
* Multiple entities can use it to get the reference to their imagesprite. 
* Note that only one imagesprite exists of every texture and that moving one of them will move them all.
* 
* The enum IMG_ID:s are used since they are easier and more effective to 
* compare against throughout the system, even if it brings more overhead. 
* It's also to keep file path private only for this ImageSubSystem so that the rest of the 
* system don't need to know them nor compare char arrays in order to identify their graphical resource. 
* It also makes it impossible for other potential programmers to just hard code a string path 
* directly in an entity class, and problems will be spotted compile-time.
*
* Only one instance should exist of this class but it should NOT be globally available, instead only accessed 
* through the GraphicsSystem facade (which is why this class is not a singleton). 
* There could instead be argues that only GraphicsSystem should be a friend class that could access 
* and create an instance of this class, but that would break encapsulation and open up access 
* also to other private functions and variables to GraphicsSystem (functions it should not be able to use)
**/
class ImageSubSystem
{
public:

	// Instantiates
								ImageSubSystem(); 

	// Cleans up all textures, deletes imagesprites, clears map of imagesprites
								~ImageSubSystem(); 

	// Loads textures into imagesprites and puts them in map
	// Returns false if some image was not added
	bool 						Init();

	// Gives out a reference to the imagesprite with given id 
	// Tries to recover and load the imagesprite if it could not be found
	// Finally throws an error message if the sprite still could not be retrieved
	// Retrieved ImageSprites should NOT be deallocated externally, will instead
	// be taken care of automatically at destruction of the ImageSubSystem
	ImageSprite&				GetImage(IMG_ID id) throw (const char*);


private:

	// Instantiates, initiates and loads all sprites 
	bool						LoadImages();

	// Fills map with sprites and their ids, returns false if any sprite failed
	bool						AddImages();

	// Adds a new sprite and maps it with id. ID must be unique, or it will return false 
	bool						AddImage(IMG_ID mapKeyId);

	// Deletes textures & sprites 
	void						UnloadImages();

	// map that stores sprites references and their ID
	std::map<IMG_ID, ImageSprite*> m_id_to_sprites; 
};