#include "PlayState.h"
#include "ScreenManager.h"
#include "SoundSystem.h"
#include "GraphicsSystem.h"
#include "Timer.h"
#include "Galaxy.h"
#include "Player.h" 
#include "LevelManager.h"
#include "HUD.h"
#include <stdio.h>

PlayState::PlayState(StateKey stateKey, Timer& timer):
BaseState(stateKey, timer)
{
	m_galaxy = new Galaxy(); // Inits world and objects
	m_player = new Player(*m_galaxy); // The player controlled ship
	m_levelManager = new LevelManager(); // Level 
	m_HUDManager = new HUD();

	m_bIsGamePaused = false;
}


PlayState::~PlayState()
{
	delete m_HUDManager;
	m_HUDManager = NULL;

	delete m_levelManager;
	m_levelManager = NULL;

	delete m_player;
	m_player = NULL;

	delete m_galaxy;
	m_galaxy = NULL;
	
	// shadow reference that is not for this class to delete
	
}

void PlayState::BeginState()
{
	printf("INFO: Beginning PlayState\n");
	ResetBeforeGame();
	ResetBeforeLevel(); 
	ShowCountDownScreen(m_levelManager->GetLevelText());
	SoundSystem::GetSingleton()->PlayBGMusicForLevel(m_levelManager->GetCurrentLevel());
	m_timerRef.SetNewTimerAndGetElapsedTime();
}

void PlayState::IncreaseLevel()
{
	m_levelManager->IncreaseLevel();
	ResetBeforeLevel(); 

	m_galaxy->IncreaseRockSpeed();

	ScreenManager::GetSingleton()->ShowTextOnScreen(m_levelManager->GetLevelText());
	m_timerRef.WaitSeconds(2.5f); 

	SoundSystem::GetSingleton()->PlayBGMusicForLevel(m_levelManager->GetCurrentLevel());
	printf("INFO: Starting new level: %d\n", m_levelManager->GetCurrentLevel());

	m_timerRef.SetNewTimerAndGetElapsedTime();
}

void PlayState::Update(std::vector<Action*>& actionsRef)
{
	if (m_bIsGamePaused)
	{
		HandlePauseSubState(actionsRef);
	}
	else
	{
		HandlePlaySubState(actionsRef);
	}
}


void PlayState::HandlePlaySubState(std::vector<Action*>& actionsRef)
{
	if (m_bGameOver)
		return;

	float elapsedTime;

	if ( !m_levelManager->HasReachedMaxCounter()) // level loop while game is not finished and the player has not quit
	{			
		elapsedTime = m_timerRef.SetNewTimerAndGetElapsedTime();

		m_galaxy->Update(elapsedTime, m_levelManager->IsStarting());
		m_HUDManager->Update(m_levelManager->GetPercentOfLevelDone());
		m_player->Update(elapsedTime, actionsRef); 
		m_galaxy->HandleCollisions(*m_player);

		if (!m_player->IsAlive()) // player died
		{
			EndGame();
		}

		if (m_galaxy->IsAllowedToSpawnRocks() && m_levelManager->IsEnding()) // stops spawn rocks at end of level
		{
			m_galaxy->StopSpawnRocks();
			SoundSystem::GetSingleton()->FadeOutMusic(false);
		}

		m_galaxy->Draw();
		m_player->Draw();
		m_HUDManager->Draw(m_player->GetNumberOfLives());
		GraphicsSystem::GetSingleton()->ShowBufferedScreenOnDisplay();	// shows everything that was drawn
				
		m_levelManager->IncreaseCounter(elapsedTime);		// increases counter controlling how many loops per level			
			


	}
	else  // level loop ended, time to loop again (next level) or quit 
	{
		EndLevel();	
	}

	for (size_t i = 0; i < actionsRef.size(); i++) 
	{
		if(*(actionsRef[i]) == A_PAUSE)
			EnterPause();
	}
}

void PlayState::EndGame()
{
	SoundSystem::GetSingleton()->FadeOutMusic(false);
	ScreenManager::GetSingleton()->ShowTextOnScreen("GAME OVER");
	m_timerRef.WaitSeconds(3.0f);			 
	m_bGameOver = true;
}

void PlayState::WinGame()
{
	ScreenManager::GetSingleton()->ShowTextOnScreen("You won!");
	m_timerRef.WaitSeconds(4.0f);
	m_bGameOver = true; 
}

void PlayState::EndLevel()
{
	if (m_levelManager->HasReachedMaxCounter())
	{
		ShowShipFlyOutAnimation(); // Animation to fly ship out faster and faster

		if (m_levelManager->HasReachedMaxLevel()) 
		{
			WinGame();
		}
		else
		{
			IncreaseLevel();
		}
	}
}



void PlayState::ShowCountDownScreen(const char* levelText) 
{
	ScreenManager::GetSingleton()->ShowTextOnScreen(levelText, "Starting in... 3", false); 
	m_timerRef.WaitSeconds(1.0f);

	ScreenManager::GetSingleton()->ShowTextOnScreen(levelText, "Starting in... 2", false);
	m_timerRef.WaitSeconds(1.0f);
	
	ScreenManager::GetSingleton()->ShowTextOnScreen(levelText, "Starting in... 1", false);
	m_timerRef.WaitSeconds(1.0f);
}

void PlayState::ResetBeforeGame()
{
	m_HUDManager->ResetBeforeGame();
	m_galaxy->ResetBeforeGame();
	m_player->ResetBeforeGame();
	m_levelManager->ResetBeforeGame();
	m_bGameOver = false;
}

void PlayState::ResetBeforeLevel()
{
	m_HUDManager->ResetBeforeLevel(m_levelManager->GetCurrentLevel());
	m_levelManager->ResetBeforeLevel();		
	m_galaxy->ResetBeforeLevel(m_levelManager->GetCurrentLevel());
	m_player->ResetBeforeLevel();
}

void PlayState::ShowShipFlyOutAnimation()
{
	m_timerRef.SetNewTimerAndGetElapsedTime(); // resetting
	float elapsedTime;

	while(m_player->IsPlayerInThePicture())
	{
		elapsedTime = m_timerRef.SetNewTimerAndGetElapsedTime();
		m_player->RunPlayerForwardAnimation();
		m_galaxy->Update(elapsedTime, false); // need last stones to move out
		m_galaxy->Draw();
		m_player->Draw();
		GraphicsSystem::GetSingleton()->ShowBufferedScreenOnDisplay();		
	}
}

StateKey PlayState::DesiredStateChange() const
{
	if (m_bGameOver)
		return STATE_MENU; 
		
	return STATE_NONE;
}

void PlayState::HandlePauseSubState(std::vector<Action*>& actions)
{
	for (size_t i = 0; i < actions.size(); i++)
	{
		if (*(actions[i]) == A_PAUSE)
			ExitPause();
	}
	
	m_timerRef.WaitSeconds(0.1f); // Don't poll input 100% of the time
}

void PlayState::EnterPause()
{
	m_bIsGamePaused = true;
	ScreenManager::GetSingleton()->ShowTextOnScreen("PAUSED", "Press [Pause] to play", true);
	SoundSystem::GetSingleton()->PauseMusic();
}

void PlayState::ExitPause()
{
	m_bIsGamePaused = false;
	SoundSystem::GetSingleton()->ResumeMusic();
	m_timerRef.SetNewTimerAndGetElapsedTime();
}

void PlayState::EndState()
{
	SoundSystem::GetSingleton()->FadeOutMusic(true); // case it's not already fading out, it should do so quickly 
	printf("INFO: Ending PlayState\n");
}