#include "Player.h"
#include "ImageConfig.h" // IMG_ID
#include <cmath> // abs
#include "GraphicsSystem.h"
#include "RoundEntity.h"
#include "Galaxy.h"
#include "ImageSprite.h"

// Constants:
enum {  
	SPEED_PLAYER = 4,
	START_LIVES = 3,
	BLINK_COUNTER_MAX = 150,
	FIRE_OVERLAP = 30,
	FIRE_RELATIVE_Y = 20,
	POLISH_TWEAK_OFFSET = 15
};

enum HITBOX {
	SHIP_FRONT_UPPER_X = 95,
	SHIP_FRONT_UPPER_Y = 23,
	SHIP_FRONT_LOWER_X = 103,
	SHIP_FRONT_LOWER_Y = 32,
	SHIP_BACK_UPPER_X = 22,
	SHIP_BACK_UPPER_Y = 20,
	SHIP_BACK_LOWER_X = 38,
	SHIP_BACK_LOWER_Y = 48,
	SHIP_CENTER_X = 61,
	SHIP_CENTER_Y = 22,
};

const float PLAYER_FORWARD_ANIM_SPEED_ACCELERATION = 1.03f;

Player::Player(Galaxy& galaxyRef)
:Entity(GraphicsSystem::GetSingleton()->GetImage(IMG_ID_SHIP)),
m_fireImgSpriteRef(GraphicsSystem::GetSingleton()->GetImage(IMG_ID_FIRE)),
m_galaxyRef(galaxyRef)
{
}

Player::~Player()
{	
}

void Player::ResetBeforeGame()
{
	m_lives = START_LIVES;
}

void Player::ResetBeforeLevel()
{
	SetPositionX(PADDING);			// player start pos x
	SetPositionY( (float) SCREEN_HEIGHT / 2.0f - (float) GetHeight() / 2.0f);	// player start pos y in middle
	m_blinkCounter = 0.0f;	//reset blinking
	SetOpacity(1.0f);
	m_playerForwardAnimationSpeed = (float) SPEED_PLAYER;
	m_bFireShouldBeDrawn = false;
	m_blinkCounter = 0.0f;
}

void Player::MoveAccordingToActions(float elapsedTime, std::vector<Action*>& actionsRef)
{
	float distance = SPEED_PLAYER * GAME_SPEED * elapsedTime;

	// Phase one: first processes actions to make sure only one move is taken in each direction this update
	m_verticalMovement = FLY_V_IDLE; // default move
	m_horizontalMovement = FLY_H_IDLE; // default move

	for(size_t i = 0; i < actionsRef.size(); i++) // 
	{
		switch(*(actionsRef[i])) {
			case A_UP:
				m_verticalMovement = MOVE_UP; break;
			case A_DOWN:
				 m_verticalMovement = MOVE_DOWN; break;
			case A_FLY_V_IDLE:
				m_verticalMovement = FLY_V_IDLE; break;
			case A_LEFT:
				m_horizontalMovement = MOVE_BACK; break;
			case A_RIGHT: 
				m_horizontalMovement = MOVE_FORWARD; break;
			case A_FLY_H_IDLE:
				m_horizontalMovement = FLY_H_IDLE; break;
			default: break; // not supported action
		}
	}

	// Phase two: actual movement
	switch(m_verticalMovement)
	{
		case MOVE_UP: 
		{
			MoveUp(distance); 
			break;
		}
		case MOVE_DOWN: 
		{
			MoveDown(distance); 
			break;
		}
		default:
			break;
	}
	switch(m_horizontalMovement)
	{
		case MOVE_BACK:
		{
			MoveBack(distance);
			break;
		}
		case MOVE_FORWARD:
		{
			MoveForward(distance);
			break;
		}
		default: 
			break;
	}

}

void Player::Update(float elapsedTime, std::vector<Action*>& actionsRef)
{
	MoveAccordingToActions(elapsedTime, actionsRef);

	UpdateFireVFX();
	
	// Counting down invulnerability case it's on
	if (m_blinkCounter > 0.0f) // if player sprite should be blinking
		m_blinkCounter-= GAME_SPEED * elapsedTime;

}

void Player::UpdateFireVFX()
{
		// Setting bool and position if fire should be drawn: if ship is not moving backwards and not standing still
	if (m_horizontalMovement != MOVE_BACK && !(m_verticalMovement == FLY_V_IDLE && m_horizontalMovement == FLY_H_IDLE))
	{
		m_bFireShouldBeDrawn  = true;
		m_fireImgSpriteRef.SetScreenX(GetPositionX() + (float) FIRE_OVERLAP - (float) m_fireImgSpriteRef.GetWidth());
		m_fireImgSpriteRef.SetScreenY(GetPositionY() + FIRE_RELATIVE_Y);
	}
	else m_bFireShouldBeDrawn = false;
}

void Player::MoveUp(float distance)
{
	float newPosY = GetPositionY() - distance;

	if (newPosY > 0.0f - POLISH_TWEAK_OFFSET)
	{
		SetPositionY(newPosY); 
	}
}

void Player::MoveDown(float distance)
{
	float newPosY = GetPositionY() + distance;
	float newLowerPosY = newPosY + GetHeight();

	if (newLowerPosY < SCREEN_HEIGHT + POLISH_TWEAK_OFFSET)// bounds check
	{
		SetPositionY(newPosY);
	}
}

void Player::MoveForward(float distance)
{
	float newPosX = GetPositionX() + distance;

	if (newPosX + GetWidth() < SCREEN_WIDTH) // bounds check
	{
		SetPositionX(newPosX); 
	}
}

void Player::MoveBack(float distance)
{
	float newPosX = GetPositionX() - distance;
	
	if (newPosX > PADDING) // bounds check. Fire should fit too
	{
		SetPositionX(newPosX);
	}
}

void Player::Draw() 
{
	if (m_blinkCounter > 0.0f) // if player sprite should be blinking
	{
		if(m_lastCounterMark - m_blinkCounter >= 10.0f) // every 10th counter step
		{
			if (GetOpacity() == 1.0f) // toggle opacity
				SetOpacity(1.0f - (m_blinkCounter/ (float) BLINK_COUNTER_MAX)); // should be more and more visible closer to end of blink
			else 
				SetOpacity(1.0f);

			m_lastCounterMark = m_blinkCounter;
		}

		if (m_blinkCounter <= 1.0f) // reset visibility if blinking mode just ended
			SetOpacity(1.0f);
	}		

	Entity::Draw();

	// Drawing fire
	if (m_bFireShouldBeDrawn) 
	{
		m_fireImgSpriteRef.SetOpacity(GetOpacity()); // same visbility as ship
		m_fireImgSpriteRef.Draw();
	}

}

bool Player::IsCloseToHitbox(const RoundEntity& otherRoundEntityRef) const
{
	// checking bounding box before 
	float distanceX = abs(GetPositionX() + SHIP_CENTER_X - otherRoundEntityRef.GetCenterX());
	float minRoughDistanceX = otherRoundEntityRef.GetRadius() + SHIP_FRONT_LOWER_X; // longest point in hitbox.
	return (distanceX  < minRoughDistanceX);
}


bool Player::IsIntersecting(const RoundEntity& otherRoundEntityRef) const
{
	float hypotenusaSqrt, distanceX, distanceY, radiusSqrt;

	if (!IsCloseToHitbox(otherRoundEntityRef)) // rough check
		return false;

	// more detailed check below
	radiusSqrt = otherRoundEntityRef.GetRadius() * otherRoundEntityRef.GetRadius();

	// Checking against first point - most common hit point - upper right corner on ship's front
	distanceX = GetPositionX() + SHIP_FRONT_UPPER_X - otherRoundEntityRef.GetCenterX();
	distanceY = GetPositionY() + SHIP_FRONT_UPPER_Y - otherRoundEntityRef.GetCenterY();
	hypotenusaSqrt = (distanceX*distanceX + distanceY*distanceY);
	if (hypotenusaSqrt < radiusSqrt) return true;

	// Checking against second point - lower right launcher on ship
	distanceX = GetPositionX() + SHIP_FRONT_LOWER_X - otherRoundEntityRef.GetCenterX();
	distanceY = GetPositionY() + SHIP_FRONT_LOWER_Y - otherRoundEntityRef.GetCenterY();
	hypotenusaSqrt = (distanceX*distanceX + distanceY*distanceY);
	if (hypotenusaSqrt < radiusSqrt) return true;

	// Checking against third point - upper back point on ship
	distanceX = GetPositionX() + SHIP_BACK_UPPER_X - otherRoundEntityRef.GetCenterX(); 
	distanceY = GetPositionY() + SHIP_BACK_UPPER_X - otherRoundEntityRef.GetCenterY();
	hypotenusaSqrt = (distanceX*distanceX + distanceY*distanceY);
	if (hypotenusaSqrt < radiusSqrt) return true;

	// Checking against fourth point - lower back launcher on ship
	distanceX = GetPositionX() + SHIP_BACK_LOWER_X - otherRoundEntityRef.GetCenterX(); 
	distanceY = GetPositionY() + SHIP_BACK_LOWER_Y - otherRoundEntityRef.GetCenterY();
	hypotenusaSqrt = (distanceX*distanceX + distanceY*distanceY);
	if (hypotenusaSqrt < radiusSqrt) return true;	

	// Checking against fifth point - center
	distanceX = GetPositionX() + SHIP_CENTER_X - otherRoundEntityRef.GetCenterX(); 
	distanceY = GetPositionY() + SHIP_CENTER_Y - otherRoundEntityRef.GetCenterY();
	hypotenusaSqrt = (distanceX*distanceX + distanceY*distanceY);
	if (hypotenusaSqrt < radiusSqrt) return true;	

	// it could potentially hit the back of the ship, if the player
	// moved fast enough. but that's quite ok. let's not make it too hard.
	// the player won't notice anyway
	return false;
}

bool Player::IsPlayerInThePicture() const
{
	if (GetPositionX() > 0.0f && GetPositionX() < SCREEN_WIDTH && GetPositionY() > 0.0f && GetPositionY() < (float) SCREEN_HEIGHT)
		return true;
	return false;
}
void Player::RunPlayerForwardAnimation()
{
	SetPositionX(GetPositionX() + m_playerForwardAnimationSpeed); 

	m_fireImgSpriteRef.SetScreenX(GetPositionX() + FIRE_OVERLAP - m_fireImgSpriteRef.GetWidth());
	m_fireImgSpriteRef.SetScreenY(GetPositionY() + (float)GetHeight() / 2.0f - (float)m_fireImgSpriteRef.GetHeight() / 2.0f); // setting this every call is not really necessary but we should have system resources for it
	m_bFireShouldBeDrawn = true; // setting this each call is not really necessary but we should have system resources for it

	m_playerForwardAnimationSpeed *= PLAYER_FORWARD_ANIM_SPEED_ACCELERATION;
	
}
int Player::GetNumberOfLives() const
{
	return m_lives;
}

void Player::CollidedWithBonusLife()
{
	 m_lives++; // collected bonus life
}
bool Player::CanCollideWithRocks() const
{
	return m_blinkCounter <= 0.0f; // means not in blinking (invunerable) mode
}
void Player::CollidedWithRock()
{
	m_blinkCounter = BLINK_COUNTER_MAX; // will be invunerable and blink for a short time
	m_lastCounterMark = m_blinkCounter; // start value
	m_lives--;
}

bool Player::IsAlive() const
{
	return m_lives > 0;
}