#pragma once //microsoft-specific

#include "Entity.h"
#include "ICollidableEntity.h"
#include "Globals.h" // Action
#include <vector>

// forward declaration to speed up compilation
class Galaxy; 
class RoundEntity; 

/**
 * Represents the player. Has a ship imagesprite (encapsulated in base class),
 * and a fire imagesprite. Draws them both. 
 * Takes movement actions that were pre-set by statemanager.
 * 
 * Passive class, controlled by input actions and galaxy.
 * Has collision detection functions for round entities (bonus life and rocks).
 * Player gains life if colliding with a bonus life.
 * If the player hit a rock, it will lose a life and become
 * invulnerable for a short time (blinking animation).
 **/
class Player:
	public Entity, public ICollidableEntity
{
public:
								Player(Galaxy& galaxyRef);
								~Player();

	// Updates ship movements and VFX according to actions given
	void 						Update(float elapsedTime, std::vector<Action*>& actions);
	void 						Draw();

	// Returns true if the ship is still visible within the game window. 
	bool 						IsPlayerInThePicture() const; 

	// Makes the ship go faster and faster in forward direction. Call continously.
	// Not time bound e.g. in a high end PC it will go faster
	void 						RunPlayerForwardAnimation();

	// Returns the current number of lives remaining
	int 						GetNumberOfLives() const;

	// Checks collision with roundentity radius, compared to a few points on ship
	// Optimized for most common hit points
	bool 						IsIntersecting(const RoundEntity& otherRoundEntityRef) const; 

	void 						ResetBeforeGame();
	void 						ResetBeforeLevel();

	// Increases the number of lives
	void 						CollidedWithBonusLife();

	// Tells if the player is vulnerable
	bool 						CanCollideWithRocks() const;

	// Looses a life
	void 						CollidedWithRock(); 

	// Tells if the player lost all lives or not
	bool 						IsAlive() const;

private:
	
	enum VerticalMovement
	{
								MOVE_UP,
								MOVE_DOWN,
								FLY_V_IDLE
	};

	enum HorizontalMovement 
	{
								MOVE_BACK,
								MOVE_FORWARD,
								FLY_H_IDLE
	};

	// Moves the ship according to user input filtrated into actions
	void 						MoveAccordingToActions(float distance, std::vector<Action*>& actions);

	void 						MoveUp(float distance);
	void 						MoveDown(float distance);
	void 						MoveForward(float distance);
	void 						MoveBack(float distance);

	// shallow collision check
	bool 						IsCloseToHitbox(const RoundEntity& otherRoundEntityRef) const; 

	// Setting bool and position if fire should be drawn: if ship is not moving backwards and not standing still
	void 						UpdateFireVFX();

	Galaxy& 					m_galaxyRef;
	ImageSprite&				m_fireImgSpriteRef;

	// decides which one step should be taken each update vertically
	enum VerticalMovement		m_verticalMovement;

	// decides which one step should be taken each update horizontally
	enum HorizontalMovement		m_horizontalMovement;

	int 						m_lives;
	float						m_blinkCounter;
	bool 						m_bFireShouldBeDrawn;
	float 						m_playerForwardAnimationSpeed;
	float 						m_lastCounterMark;



};

