#pragma once //microsoft-specific

#include "RoundEntity.h"

// forward declaration to speed up compilation
class ImageSprite; 

/**
 * Represents a rock in the game. Has functions for collision detection.
 * Rocks move from right to left, and spawns at random interval with 
 * some space in between. Tries to spawn by itself automatically in its update function.
 * If no space was found, waits longer to try to spawn. 
 * Only RockFactory may create rocks, which is why it's a friend class with 
 * private constructor.
 **/
class Rock :
	public RoundEntity
{
public:	
	~Rock();
	void Update(float elapsedTime);
	void ResetBeforeLevel();
	float GetCenterX() const; // overriding. the center should be the center of the HITBOX not ImageSprite texture
	float GetCenterY() const;	// overriding. the center should be the center of the HITBOX, not ImageSprite texture
	float GetRadius() const; 	// overriding. the radius should be the center of the HITBOX, not ImageSprite texture
	
	void Draw();

	// Increase difficulty
	static void IncreaseRockSpeed();

	// For each game, call this to make sure the speed is set to its initial value.
	static void ResetRockSpeed();

	// stops the rocks from spawnning. Typically called at end of a level.
	static void StopSpawn();

	// Sets flag that the rocks may try to spawn. Typically at start of every level until the end.
	static void AllowSpawn();

	// Tells if the rocks are currently allowed to spawn or not
	static bool IsAllowedToSpawn();

	// For performance, the rocks that are trying to spawn share a common counter
	// that says how many collective tries they have to find a free spot to spawn on
	// no point for all rocks to try spawn where those before have failed many times.
	static void ResetNrOfTrySpawnAttempts(bool bShouldOnlyTrySpawnHalf);

private:

	// Only RockFactory may create rocks. Private constructor marks this.
	Rock(ImageSprite& imageSprite, int hitBoxDiameter, Galaxy& galaxyRef);

	// Overloading base class with special parameters
	void TrySpawn();

	static float m_rockSpeed;
	int m_hitBoxDiameter;
	float m_rotationSpeed;
	float m_rotationDegree;
	static bool m_bIsAllowedToSpawn;
	static short m_nrRemainingRockTrySpawnAttempts; // global

	// Only RockFactory may create rocks. Friend class. Breaks encapsulation but makes sure the rocks are initiated in a consistent way
	friend class RockFactory;
};