#pragma once //microsoft-specific

#include "Entity.h"
#include "IIntersectableEntity.h"

class Galaxy; // cross reference

/**
 * Base class for BonusLife and Rock. Contains their common behavior,
 * namely for spawning and intersection detection.
 * Has a counter that indicates when this RoundEntity should spawn.
 * A spawn counter above zero indicates it should not spawn yet.
 * Round entities should manually count down and all call TrySpawn() 
 * in their update functions when the counter reaches zero. 
 * Countdown functionality should be related to game speed.
 * If TrySpawn does not manage to spawn, the counter will be 
 * set again to a random value according to set intervals.
 **/
class RoundEntity :
	public Entity, public IIntersectableEntity
{
public:
								RoundEntity(ImageSprite& imgSprite, const Galaxy& galaxyRef);
	virtual						~RoundEntity();
	bool 						IsIntersecting(const RoundEntity& otherRoundEntityRef) const;

	// Sets this RoundEntity to not be alive
	void 						Unspawn();

	virtual float 				GetRadius() const;

	// Tells if this RoundEntity is alive (in play/active) or not
	bool 						IsAlive() const;

protected:
	// Tries to spawn on the specified place for the specified amount of times.
	// If no space was found, it sets random spawn timer again = waits longer to try to spawn. 
	void 						TrySpawn(int maxTries, float minSpawnTime, int rangeX, int rangeY, float minPosX, float minPosY);

	// The minimum distance that must be between this round entity and other in 
	// intersection detection
	void 						SetMinDistance(float minDistance);

	// Sets the spawn timer to a random value above the specified time and the spawn interval. 
	// A counter above zero indicates it is not time to spawn yet.
	void 						SetRandomSpawnTimer(float minSpawnTime); 

	// Sets the length of the window in which this round entity can be set to spawn
	void 						SetSpawnInterval(int interval);

	// Counter functions for setting and giving values on when it's time to spawn.
	// Should be relative to game speed and counted down in the update functions
	float 						GetSpawnCounter() const;
	void 						SetSpawnCounter(float spawnCounter);

	const Galaxy& 					m_galaxyRef;

private:

	// Avoid assign of this class, it should not be used in that way
	RoundEntity& 				operator=(const RoundEntity& );

	// Mirrors and says if this RoundEntity is onscreen and active in game or not
	bool 						m_bIsAlive;
	float 						m_minDistance;
	float 						m_spawnCounter;

	// How long the spawn window is in counts
	int 						m_spawnInterval;
};

