#include "SoundSystem.h"
#include <SDL.h>
#include "AssertButOnlyInDebug.h"

// X-macro for mapping sound resources (enum) together with its path string. 
// Defining the paths below. The rest of the Macro is in SoundConfig.h
// This part of the code is placed here to be defined and compiled only once
#define X(a, b) b,

	const char *path_for_music[] = {
	  MUSIC_ID_AND_PATH_TABLE
	};

	const char *path_for_sfx[] = {
	  SFX_ID_AND_PATH_TABLE
	};

#undef X



enum VOLUME_PERCENTAGE_OF_MAX {
	VOLUME_SFX = 50,
	VOLUME_MUSIC = 16
};

enum {
	MUSIC_FADE_OUT_MS_QUICK = 1500,
	MUSIC_FADE_OUT_MS_SLOW = 3500
};

// Re-declaring to get handle for static instance 
SoundSystem* SoundSystem::ms_soundSystem = NULL; 

SoundSystem* SoundSystem::GetSingleton()
{
	if (ms_soundSystem == NULL)
		ms_soundSystem = new SoundSystem();		

	return ms_soundSystem;
}

SoundSystem::SoundSystem()
{
	m_bIsInitialized = false;
	m_lastSFXPlayedId = -1;
	m_music = NULL;
	m_SFX = NULL;
}


SoundSystem::~SoundSystem()
{
	printf("INFO: Destroying SoundSystem\n");
	if (m_music != NULL) 
	{
		Mix_FreeMusic(m_music); // halts the music first
		m_music = NULL;
	}

	if (m_SFX != NULL) 
	{	
		Mix_FreeChunk(m_SFX); // halts the music first
		m_SFX = NULL;
	}

	if (m_bIsInitialized) // no-fail guarantee for destructor
	{
		Mix_HaltChannel(-1); // stop all channelns		
		Mix_AllocateChannels(0); // stop and free all channels
		Mix_CloseAudio();
	}
}

void SoundSystem::Destroy()
{
	if(ms_soundSystem != NULL) 
	{
		delete ms_soundSystem;
		ms_soundSystem = NULL;
	}
}

bool SoundSystem::Init()
{
	m_bIsInitialized = InitAudio();
	return m_bIsInitialized;
}

bool SoundSystem::InitAudio() 
{
	// Init SDL
	if ( SDL_Init(SDL_INIT_AUDIO ) )
    {
		printf("ERROR: %s: Failed to initialize SDL audio: %s\n", __FUNCTION__, SDL_GetError()); // microsoft-specific
        return false;
    }

    // Create handle to audio device
	int audioRate = 22050; // sampling rate
	Uint16 audioFormat = AUDIO_S16; // 16-bit stereo
	int audioChannels = 2;
	int audioBuffers = 4096; // buffer size for output

	if( Mix_OpenAudio(audioRate, audioFormat, audioChannels, audioBuffers) != NULL) 
	{
		printf("ERROR: %s: Failed to init sound system with the following parameters: audio rate: %d, audio format: %d, audio channels: %d, audio buffers: %d \n", __FUNCTION__, audioRate, audioFormat, audioChannels, audioBuffers); // microsoft-specific
		return false;
	}
	printf("INFO: Successfully created handle to audio device\n");

	// Number of channelns for sound effects, not save as audioChannels
	int nr_sfx_channels = sizeof(path_for_sfx) / sizeof (path_for_sfx[0]);
	Mix_AllocateChannels(nr_sfx_channels);

	return true;
}

void SoundSystem::PlayMusic() 
{
	if (!m_bIsInitialized)
		return;

	if(Mix_PlayingMusic()) // only one music at the time
		Mix_HaltMusic();

	if (m_music != NULL)
		Mix_PlayMusic(m_music, 0);
}

void SoundSystem::PlayBGMusicForLevel(int level)
{
	if (!m_bIsInitialized)
		return;

	assert(level > 0 && "SoundSystem::PlayBGMusicForLevel: Level must be at least one to play music");
	assert(level <= sizeof(path_for_music)/sizeof(path_for_music[0]) && "SoundSystem::PlayBGMusicForLevel: Cannot play music: there is no music for this level");
	LoadMusic(path_for_music[level-1]); // array index starts with 0 while level starts with 1
	Mix_VolumeMusic(MIX_MAX_VOLUME * VOLUME_MUSIC / 100); // background music volume (gets rounded down)
	PlayMusic();
}

void SoundSystem::PlaySFX(SFX_ID id)
{
	if (!m_bIsInitialized)
		return;

	const char* url = path_for_sfx[id];
	if (m_lastSFXPlayedId != id)
	{
		LoadSFX(url); 
		m_lastSFXPlayedId = id;
	}

	if (m_SFX == NULL || Mix_PlayChannel(id, m_SFX, 0) == -1)
	{
		printf("ERROR: %s: Failed to play sound effect: %s, error message: %s\n", __FUNCTION__, url, Mix_GetError()); // microsoft-specific	
	}

	Mix_VolumeChunk(m_SFX, MIX_MAX_VOLUME * VOLUME_SFX / 100); // sfx volume (gets rounded down)
}

void SoundSystem::FadeOutMusic(bool bQuick) 
{
	if (!m_bIsInitialized)
		return;

	if (Mix_PlayingMusic() && !Mix_PausedMusic())
		Mix_FadeOutMusic(bQuick? MUSIC_FADE_OUT_MS_QUICK : MUSIC_FADE_OUT_MS_SLOW);
}
void SoundSystem::LoadMusic(const char* url)
{
	if (!m_bIsInitialized)
		return;

	// Freeing old sound. it will halt first.
	if (m_music != NULL)
		Mix_FreeMusic(m_music); 

	m_music = Mix_LoadMUS(url);
	if (m_music == NULL) // don't break the game just because there is no sound
	{
		printf("ERROR: %s: Failed to load music: %s, error message: %s\n", __FUNCTION__, url, Mix_GetError()); // microsoft-specific
		return;
	}
	printf("INFO: Loaded music: %s\n", url);
}

void SoundSystem::LoadSFX(const char* url)
{
	if (!m_bIsInitialized)
		return;

	// Freeing old sound. it will halt first.
	if (m_SFX != NULL)
		Mix_FreeChunk(m_SFX); 
	
	m_SFX = Mix_LoadWAV(url);

	if (m_SFX == NULL) // don't break the game just because there is no sound
	{
		printf("ERROR: %s: Failed to load sound effect: %s, error message: %s\n", __FUNCTION__, url, Mix_GetError()); // microsoft-specific
		return;
	}
	printf("INFO: Loaded sound effect: %s\n", url);
}

void SoundSystem::PauseMusic()
{
	if (!m_bIsInitialized)
		return;

	if (Mix_PlayingMusic() && !Mix_PausedMusic())
		Mix_PauseMusic();
}

void SoundSystem::ResumeMusic()
{
	if (!m_bIsInitialized)
		return;

	if (Mix_PlayingMusic() && Mix_PausedMusic())
		Mix_ResumeMusic();
}
void SoundSystem::StopMusic()
{
	if (!m_bIsInitialized)
		return;

	if (Mix_PlayingMusic())
		Mix_HaltMusic();

}
bool SoundSystem::IsMusicPlaying() const
{
	if (!m_bIsInitialized)
		return false;

	return Mix_PlayingMusic() > 0;
}
const char* SoundSystem::GetMusicURL(int nr) const
{
	assert(nr >= 0 && "SoundSystem::GetMusicURL(int nr):  Tried to get the title of a song with index less than zero");
	assert(nr < sizeof(path_for_music)/sizeof(path_for_music[0]) && "SoundSystem::GetMusicURL(int nr): Tried to get the title of a song with too high index");
	return path_for_music[nr];  
}