#pragma once //microsoft-specific

#include <SDL_mixer.h>
#include "SoundConfig.h"

/**
* System for sound.
* Provides control to play, pause, resume, stop or fade out music.
* Sounds are stored in SoundConfig.h in the order of the level they will be played on
* Implemented as singleton to only have one instance that is globally available.
* Always call Init() first to initialize SDL, create handle to audio device and load music.
* Clean up by calling Destroy() from same class which called Init() for consistency.
*
* If the SoundSystem was not initialized correctly, it will ignore all calls to it except Destroy()
* and GetMusicURL(). This is because the game should work even if the sound does not.
*/
class SoundSystem
{
public:
	// Returns pointer to instance of this class. Initializes instance on first call
	static 						SoundSystem* GetSingleton();

	//Initializes sound. Returns false if it failed. Prints error message to log.
	bool 						Init();

	// Stops any playing sounds, cleans up used memory
	static void 				Destroy();

	// Plays the background sound for the specified level
	void 						PlayBGMusicForLevel(int level);

	// Plays the specific sound effect
	void 						PlaySFX(SFX_ID id);

	// Starts to fade music
	void 						FadeOutMusic(bool bQuick);

	void 						PauseMusic();

	void 						ResumeMusic();

	void 						StopMusic();

	bool 						IsMusicPlaying() const;

	// Returns the const char of the song's filename. Index 0 for first song.
	const char* 				GetMusicURL(int nr) const;

private:
	// Avoid instantiation for singleton
								SoundSystem();

	// Encapsulated to only be used by Destroy() in order to delete in a consistent way.
	// Most operating systems delete static instances at program termination, 
	// but it's done here manually as good practice
								~SoundSystem();

	// Avoid copying of this class
								SoundSystem(const SoundSystem &);

	// Avoid assign of this class. Should be called through GetSingleton
	SoundSystem					&operator=(const SoundSystem &);

	// inits SDL mixer and creates handle to audio device
	bool 						InitAudio();

	// Loads music file on given path. Uses OGG, MP3 and MIDI
	void 						LoadMusic(const char* url);

	// Loads sound effect file on given path. Uses OGG, WAVE, AIFF, RIFF and VOC
	void 						LoadSFX(const char* url);

	// Plays the loaded music, if any music is loaded
	void 						PlayMusic();

	static SoundSystem* 		ms_soundSystem;
	Mix_Music*					m_music;
	Mix_Chunk*					m_SFX;
	bool						m_bIsInitialized;
	int							m_lastSFXPlayedId;
};

