#include "StarField.h"
#include "GraphicsSystem.h"
#include "ImageConfig.h"
#include "ImageSprite.h"

const float STARFIELD_NORMAL_SPEED = 0.7f;
const float STARFIELD_SLOW_SPEED = 0.5f;
const float STARFIELD_OPACITY = 0.8f;		

enum { 
		NUM_PHASES_PER_GLOWING_STARFIELD = 3, // glowing up, glowing down, pause
	   COUNTER_CYCLE_LENGTH_PER_GLOWING_STARFIELD_PHASE  = 100, // how 
	   GLOWING_STARFIELD_COUNTER_INCREASE_PER_GAME_SPEED = 2,
	   TURNING_POINT_IN_CYCLE_WHEN_GLOWING_STARFIELD_SHOULD_START_GO_FROM_VISIBLE_TO_INVISIBLE = 100,
	   TURNING_POINT_IN_CYCLE_WHEN_GLOWING_STARFIELD_SHOULD_BE_AND_STAY_INVISIBLE = 200
};

StarField::StarField():
m_glowingStarsCounter(0),
m_backgroundStarsRef(GraphicsSystem::GetSingleton()->GetImage(IMG_ID_BG_STAR_FIELD)),
m_starFieldRef(GraphicsSystem::GetSingleton()->GetImage(IMG_ID_STAR_FIELD))
{

	IMG_ID glowingStarsID[] = { IMG_ID_GLOWING_STARS1, IMG_ID_GLOWING_STARS2, IMG_ID_GLOWING_STARS3 };

	for (int i = 0; i < NUMBER_OF_GLOWING_STARFIELDS; i++)
		m_glowingStars[i] = &GraphicsSystem::GetSingleton()->GetImage(glowingStarsID[i]);
	
	m_starFieldRef.SetOpacity(STARFIELD_OPACITY);

	m_glowingStarsFullCycleCounterMax = NUMBER_OF_GLOWING_STARFIELDS *  NUM_PHASES_PER_GLOWING_STARFIELD * COUNTER_CYCLE_LENGTH_PER_GLOWING_STARFIELD_PHASE;
}


StarField::~StarField()
{
	for (int i = 0; i < NUMBER_OF_GLOWING_STARFIELDS; i++)
		m_glowingStars[i] = NULL;
	
}

void StarField::Update(float elapsedTime)
{
	//Moving all starsstars
	for (int i = 0; i < NUMBER_OF_GLOWING_STARFIELDS; i++)
		m_glowingStars[i]->SetScreenX(m_glowingStars[i]->GetScreenX() - STARFIELD_NORMAL_SPEED * GAME_SPEED * elapsedTime);

	m_starFieldRef.SetScreenX(m_starFieldRef.GetScreenX() - STARFIELD_NORMAL_SPEED * GAME_SPEED * elapsedTime);
	m_backgroundStarsRef.SetScreenX(m_backgroundStarsRef.GetScreenX() - STARFIELD_SLOW_SPEED  * GAME_SPEED * elapsedTime);
	
	 // if front/middle stars has gone out of bounds
	if(m_starFieldRef.GetScreenX() < 0.0f - (float) m_starFieldRef.GetWidth()) // all moving with same speed and start position, so they can all be tested against one of them
	{
		m_starFieldRef.SetScreenX( m_starFieldRef.GetScreenX() + m_starFieldRef.GetWidth() );

		for (int i = 0; i < NUMBER_OF_GLOWING_STARFIELDS; i++)
			m_glowingStars[i]->SetScreenX( m_glowingStars[i]->GetScreenX() + m_glowingStars[i]->GetWidth());
	}

	// if background stars (with another speed and another coordinates) are out of bounds
	if (m_backgroundStarsRef.GetScreenX() + (float) m_backgroundStarsRef.GetWidth() < 0.0f) // moving slower and should be checked separately
		m_backgroundStarsRef.SetScreenX(m_backgroundStarsRef.GetScreenX() + m_backgroundStarsRef.GetWidth()); 

	float glowingStarsCounterIncreaseThisUpdate = GLOWING_STARFIELD_COUNTER_INCREASE_PER_GAME_SPEED * GAME_SPEED * elapsedTime;

	m_glowingStarsCounter = fmod((m_glowingStarsCounter + glowingStarsCounterIncreaseThisUpdate), m_glowingStarsFullCycleCounterMax); // after this, counter ranges 0 - 899

}
void StarField::Draw() 
{
	// Implementation currently relying that the (globally available) textures have not been moved elsewhere in the program.
	// If this was a project with several developers, I would likely make private variables storing the coordinates, i.e. inside a struct with their imagesprite

	// Drawing far background stars
	float initialPosXForBG = m_backgroundStarsRef.GetScreenX();
	m_backgroundStarsRef.Draw();
	m_backgroundStarsRef.SetScreenX(initialPosXForBG + (float) m_backgroundStarsRef.GetWidth()); //drawing the overlap to the right
	m_backgroundStarsRef.Draw();

	// Drawing static starfield
	float initialPosXForStarFields = m_starFieldRef.GetScreenX();	
	m_starFieldRef.Draw();
	m_starFieldRef.SetScreenX( initialPosXForStarFields + (float) m_starFieldRef.GetWidth() ); //drawing the overlap to the right
	m_starFieldRef.Draw();


	// Drawing glowing stars
	float opacityInCurrentSFCurrentPhase = fmod(m_glowingStarsCounter, COUNTER_CYCLE_LENGTH_PER_GLOWING_STARFIELD_PHASE); // 0 - 99 
	opacityInCurrentSFCurrentPhase /= 100; // 0-0.99, to get percantage

	float totalLengthCyclePerStarField = NUM_PHASES_PER_GLOWING_STARFIELD * COUNTER_CYCLE_LENGTH_PER_GLOWING_STARFIELD_PHASE;

	int currentStarField = (int) m_glowingStarsCounter/ (int) totalLengthCyclePerStarField; // 0 - 2, index of which one is currently displayed

	float placeWhereCurrentStarFieldIsInItsCycle = fmod(m_glowingStarsCounter,totalLengthCyclePerStarField);

	if (placeWhereCurrentStarFieldIsInItsCycle < TURNING_POINT_IN_CYCLE_WHEN_GLOWING_STARFIELD_SHOULD_BE_AND_STAY_INVISIBLE){ // draw star phase. its split in 3 parts: opacity goes up, then down, then stays invisible
		if (placeWhereCurrentStarFieldIsInItsCycle < TURNING_POINT_IN_CYCLE_WHEN_GLOWING_STARFIELD_SHOULD_START_GO_FROM_VISIBLE_TO_INVISIBLE) 
			m_glowingStars[currentStarField]->SetOpacity (opacityInCurrentSFCurrentPhase); 
		else 
			m_glowingStars[currentStarField]->SetOpacity (1.0f - opacityInCurrentSFCurrentPhase); // opacity decreases, goes invisible

		m_glowingStars[currentStarField]->Draw();
		m_glowingStars[currentStarField]->SetScreenX( (initialPosXForStarFields + (float) m_glowingStars[currentStarField]->GetWidth())); 
		m_glowingStars[currentStarField]->Draw();
	} // else pause 


	// Resetting positions for all starfields
	for (int i = 0; i < NUMBER_OF_GLOWING_STARFIELDS; i++)
		m_glowingStars[i]->SetScreenX(initialPosXForStarFields);

	m_starFieldRef.SetScreenX(initialPosXForStarFields);
	m_backgroundStarsRef.SetScreenX(initialPosXForBG);

}