#include "TextSubSystem.h"
#include "MathUtil.h"
#include <fstream>
#include "FontConfig.h"
#include "AssertButOnlyInDebug.h"

enum { 
		INPUT_TEXT_BUFFER_SIZE = 200 
};

// X-macro for mapping text resources (enum) together with its path string. 
// Defining the paths below. The rest of the Macro is in TextConfig.h
// This part of the code is placed here to be defined and compiled only once
#define X(a, b) b,
	const char *path_for_text_id[] = {
	  TEXT_ID_AND_PATH_TABLE
	};
#undef X


TextSubSystem::TextSubSystem() 
{
	m_mediumFont = NULL;
	m_bigFont = NULL;
	m_smallFont = NULL;
}

TextSubSystem::~TextSubSystem() 
{
	printf("INFO: Destroying TextSubSystem\n");
	UnloadFonts();
	TTF_Quit(); 

	while(!m_allocated_texts.empty()) // for each vector or const char arrays*
	{
		while(!m_allocated_texts.back().empty()) // empty the char arrays
		{
			// casting away const in this case is safe since we know the data is indeed
			// not const since it was allocated by this same class 
			free(const_cast<char*> (m_allocated_texts.back().back())); 
			m_allocated_texts.back().pop_back();
		}
		m_allocated_texts.back().shrink_to_fit(); //  release capacity pointers in this vector.  c++11, non-binding request
		m_allocated_texts.pop_back(); // pop vector
	}
	m_allocated_texts.shrink_to_fit(); // release capacity. c++11, non-binding request
}

bool TextSubSystem::Init()
{	
	// Setting color masks for drawing texts
	// Depends on in which order the bytes with data is stored is memory
	#if SDL_BYTEORDER == SDL_BIG_ENDIAN // Mac
		m_mask_r  = 0xff000000;
		m_mask_g  = 0x00ff0000;
		m_mask_b  = 0x0000ff00;
		m_mask_a  = 0x000000ff;
		m_bColorIsRGBA = true;
	#else							// intel for windows
		m_mask_r  = 0x000000ff;
		m_mask_g  = 0x0000ff00;
		m_mask_b  = 0x00ff0000;
		m_mask_a  = 0xff000000;
		m_bColorIsRGBA = false;
	#endif

	 if (!LoadFonts())
		return false;
	return true;
}

bool TextSubSystem::LoadFonts()
{
	if (TTF_Init() == -1)
    {
		printf("ERROR: %s: Failed to init TTF: %s\n", __FUNCTION__, TTF_GetError()); //microsoft-specific
        return false;
    }
    else printf("INFO: Successfully initialized TTF\n");

	m_mediumFont = TTF_OpenFont(FONT_SPACE, FONT_SIZE_MEDIUM); 
	m_bigFont = TTF_OpenFont(FONT_SPACE, FONT_SIZE_BIG);
	m_smallFont = TTF_OpenFont(FONT_SPACE, FONT_SIZE_SMALL);

    if (m_mediumFont == NULL || m_bigFont == NULL || m_smallFont == NULL)
    {
		printf("ERROR: %s: Failed to load font: TTF_OpenFont: %s\n", __FUNCTION__, TTF_GetError()); //microsoft-specific 
        return false;
    }
    printf("INFO: Successfully loaded fonts\n");

	return true;
}

void TextSubSystem::UnloadFonts()
{
	if (TTF_WasInit())
	{
		if (m_smallFont != NULL)  
		{
			TTF_CloseFont(m_smallFont);
			m_smallFont = NULL;
		}
		if (m_bigFont != NULL)  
		{
			TTF_CloseFont(m_bigFont);
			m_bigFont = NULL;
		}
		if (m_mediumFont != NULL)  
		{
			TTF_CloseFont(m_mediumFont);	
			m_mediumFont = NULL;
		}
	}

    printf("INFO: Successfully unloaded fonts\n");
}

void TextSubSystem::DrawTextOntoScreenBuffer(const char *textToDisplay, GLfloat& posY, XAlignment alignX, Uint8& textRed, Uint8& textGreen, Uint8& textBlue, FontSize fontSize, float& opacity)
{
	assert(textToDisplay && "No text was given to draw");
	
	if (strlen(textToDisplay) == 0)
	{
		return; // nothing to draw
	}

	SDL_Color textColor;
	textColor.r = textRed;
    textColor.g = textGreen;
    textColor.b = textBlue;


	if(opacity < 0.0f) // cannot be negative
		opacity = 0.0f;
	else if (opacity > 1.0f) // max 1
		opacity = 1.0f;

	SDL_Color outlineColor;
    outlineColor.r = 0;
    outlineColor.g = 0;
    outlineColor.b = 0;






	TTF_Font* font;

	if (fontSize == FONT_SIZE_BIG) 
	{
		font = m_bigFont;	
	}
	else if (fontSize == FONT_SIZE_MEDIUM)
	{
		font = m_mediumFont; 
	}
	else
	{
		font = m_smallFont;
	}

	assert(font && "Cannot draw text: font is null and may not have been loaded");
	

    SDL_Surface *text = TTF_RenderText_Blended(font, textToDisplay, textColor);
    SDL_Surface *textOutline = TTF_RenderText_Solid(font, textToDisplay, outlineColor);

	int w = MathUtil::NextPowerOfTwo(PADDING + text->w); // some openGL drivers can only handle textures 2^x
	int h = MathUtil::NextPowerOfTwo(PADDING + text->h); // outline and text itself

    SDL_Surface *textAndOutline = SDL_CreateRGBSurface(0, w, h, 32, m_mask_r, m_mask_g, m_mask_b, m_mask_a); // putting together

	SDL_Rect rect;

    for (Sint16 y = 0; y < 5; y++)
    {
        for (Sint16 x = 0; x < 5; x++)
        {
            rect.x = x; rect.y = y;
            SDL_BlitSurface(textOutline, 0, textAndOutline, &rect); // whole content in textoutline blits to textAndOutLine
        }
    }

    rect.x = 2;
    rect.y = 2;
    SDL_BlitSurface(text, 0, textAndOutline, &rect);

		
	GLfloat posX = 0;

	if (alignX == LEFT)
    {
		posX = PADDING; // some padding first, dont start at edge
    }
    else if (alignX == CENTER)
    {
        posX = (GLfloat) (((float)SCREEN_WIDTH / 2) - ((float)text->w / 2));
    }
    else if (alignX == RIGHT)
    {
        posX = (GLfloat) ((SCREEN_WIDTH - text->w) - PADDING);
    } 

    glPushMatrix(); // pushing current matrix onto the stack
	
	GLuint textTexture;

	glGenTextures(1, &textTexture);
	glBindTexture(GL_TEXTURE_2D, textTexture);
	glTexImage2D(GL_TEXTURE_2D, 0, 4, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, textAndOutline->pixels );
 	glEnable(GL_TEXTURE_2D);
    
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);  
    
	glColor4f(1.0f, 1.0f, 1.0f, opacity);

    glBegin(GL_QUADS); // start push quads through the pipeline
        glTexCoord2f(0.0f, 1.0f);
        glVertex2f(posX    , posY + (GLfloat)h);

        glTexCoord2f(1.0f, 1.0f);
        glVertex2f(posX + (GLfloat)w, posY + (GLfloat)h);

        glTexCoord2f(1.0f, 0.0f);
        glVertex2f(posX + (GLfloat)w, posY    );

        glTexCoord2f(0.0f, 0.0f);
        glVertex2f(posX    , posY    );
    glEnd();

    glPopMatrix(); // restore the matrix from the stack

	// deletes temp objects
    SDL_FreeSurface(text);
    SDL_FreeSurface(textOutline);
    SDL_FreeSurface(textAndOutline);
    glDeleteTextures(1, &textTexture); 
}

std::vector<const char*> TextSubSystem::ReadTextFromFile(TEXT_ID id) 
{
	std::vector<const char*> texts;
	std::ifstream file;

	try // catch and deal with exceptions locally
	{
		file.open(path_for_text_id[id]); // opening for read

		if (!file.is_open()) 
		{
			throw std::exception("File could not be opened");	
		}

		while (!file.eof())   
		{
			char* line;
			line = (char*) malloc(INPUT_TEXT_BUFFER_SIZE * sizeof(char));
			file.getline(line, INPUT_TEXT_BUFFER_SIZE);
			texts.push_back(line); // char* to const char*. beware during deallocation
		} 	 
		
	}
	catch(std::exception e)
	{
		printf("ERROR: %s: Could not retrieve text from file: %s, reason: %s\n", __FUNCTION__, path_for_text_id[id], e.what()); // keep play game if the text fails. microsoft-specific
		texts.push_back("[ Missing or damaged text ]");
	}

	if (file.is_open())
		file.close();

	m_allocated_texts.push_back(texts);

	return texts;
}