 /**
 *   ____    ______      _       ______
 *  / ___\  /__  __\    / \      | __  \
 *  | |__     |  |     / _ \     | |_| |
 *  \___ \    |  |    / /_\ \    | __  /
 *   __| |    |  |   / _____ \   | | \ \
 *  /____/    |__|  /_/     \_\  |_|  \_\
 *
 * Star arcade PC game by Marie Mejerwall
 * A spaceship travels back to earth through several asteriod belts
 * (C) Ventria 2014
 * Version 5.1, 11th April 2014
 */

#define _CRTDBG_MAP_ALLOC // for memory leak checker
#include <crtdbg.h>
#include "GraphicsSystem.h"
#include "Timer.h"
#include "Input.h"
#include "StateMachine.h"
#include "SoundSystem.h"
#include "ScreenManager.h"

// THIS file is unique in having a special level of asserts: it will in fact throw error-discovery messages also in release mode to end users. 
// This is because they need some form of feedback when game crashes.
// For this reason, this file directly refers to the "assert.h" without the wrapper that only enables asserts in debug mode (that can be found inside "AssertButOnlyInDebug.h")
#include "assert.h" 


/**
* Program start entry point. 
*/
int main(int argc, char **argv)
{
	(void) argc;
	(void) argv;
	
	// Init stuff that don't fail
	Timer* timer = new Timer();	// Game clock 	
	Input* input = new Input();	// Checks user input
	StateMachine* stateMachine = new StateMachine(*timer, *input); 

	int exitStatus = EXIT_SUCCESS; // default

	try // catch-all block for the game. handled with giving user feedback below
	{
		//------------
		// INIT ENGINE
		//------------

		if (!GraphicsSystem::GetSingleton()->Init()) // Loads textures & fonts 
		{
			throw("%s: Could not initialize GraphicSystem", __FUNCTION__); // unrecoverable. microsoft-specific
		} //else:

		if (!SoundSystem::GetSingleton()->Init())
		{
			printf("ERROR: %s: The SoundSystem could not be initialized. The game will not have sound\n", __FUNCTION__); // keep play game if the sound fails to init. microsoft-specific
		} 

		//-------
		// RUN 
		//-------

		stateMachine->RunMainLoop();
		// menu loop ended, time to quit		
	}
	catch(const char* errorMsg) // something threw an exception, policy is that it must be impossible to recover from. 
	{
		printf("ERROR: Exiting program due to error: %s\n", errorMsg); // microsoft-specific
		exitStatus = EXIT_FAILURE;
			
		// User must get some form of feedback. Pressing abort or retry might not clean memory correctly. The game should continue ("Ignore") and clean resources manually in the code far below.
		assert(!"The game encountered problems and must be shut down. Please send an email to marie@ventria.se and attach the log file stdout.txt (found in the game folder). Press Ignore to quit."); 
	}


	// Cleaning up resources FILO. 	
	delete stateMachine;
	delete input;
	delete timer;
	
	// Most operating systems delete static instances at program termination, but it's done here manually as good practice
	ScreenManager::Destroy();
	SoundSystem::Destroy();
	GraphicsSystem::Destroy();  

	// Memory leak check will print in console. microsoft-specific
	_CrtDumpMemoryLeaks();

	return exitStatus;
}
